import logging

import gin

from pele.config.util import ConfigClass

# Register some logging classes with gin
gin.config.constant('ERROR', logging.ERROR)
gin.config.constant('WARNING', logging.WARNING)
gin.config.constant('INFO', logging.INFO)
gin.config.constant('DEBUG', logging.DEBUG)


@gin.configurable('logging')
class LogConfig(ConfigClass):

    def __init__(self,
                 logdir='.',
                 loglevel=logging.INFO,
                 log_format_string='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                 save_checkpoints=True,
                 checkpoint_format_string='checkpoint_epoch_{}.pt',
                 dryrun=False,
                 use_wandb=False,
                 wandb_project=None,
                 wandb_tags=None,
                 ):  # pylint:disable=too-many-arguments
        self.logdir = logdir
        self.loglevel = loglevel
        self.dryrun = dryrun
        self.log_format_string = log_format_string
        self.save_checkpoints = save_checkpoints
        self.checkpoint_format_string = checkpoint_format_string
        self.use_wandb = use_wandb
        self.wandb_project = wandb_project
        self.wandb_tags = wandb_tags if wandb_tags else []
        if self.use_wandb and not self.wandb_project:
            raise ValueError(
                'Configuration should specify logging.wandb_project is wandb is enabled')
