import inspect
import sys

import gin
import torch

from pele.config.util import ConfigClass

# Enable all of the optimizers for pytorch
gin.constant('Adadelta',torch.optim.Adadelta)  # noqa
gin.constant('Adagrad',torch.optim.Adagrad)  # noqa
gin.constant('Adam',torch.optim.Adam)  # noqa
gin.constant('Adamax',torch.optim.Adamax)  # noqa
gin.constant('AdamW',torch.optim.AdamW)  # noqa
gin.constant('ASGD',torch.optim.ASGD)  # noqa
gin.constant('LBFGS',torch.optim.LBFGS)  # noqa
gin.constant('RMSprop',torch.optim.RMSprop)  # noqa
gin.constant('Rprop',torch.optim.Rprop)  # noqa
gin.constant('SGD',torch.optim.SGD)  # noqa
gin.constant('SparseAdam',torch.optim.SparseAdam)  # noqa

# Enable the learning rate schedulers
gin.constant('CosineAnnealingLR',torch.optim.lr_scheduler.CosineAnnealingLR)  # noqa
gin.constant('CosineAnnealingWarmRestarts',torch.optim.lr_scheduler.CosineAnnealingWarmRestarts)  # noqa
gin.constant('CyclicLR',torch.optim.lr_scheduler.CyclicLR)  # noqa
gin.constant('ExponentialLR',torch.optim.lr_scheduler.ExponentialLR)  # noqa
gin.constant('LambdaLR',torch.optim.lr_scheduler.LambdaLR)  # noqa
gin.constant('MultiStepLR',torch.optim.lr_scheduler.MultiStepLR)  # noqa
gin.constant('OneCycleLR',torch.optim.lr_scheduler.OneCycleLR)  # noqa
gin.constant('ReduceLROnPlateau',torch.optim.lr_scheduler.ReduceLROnPlateau)  # noqa
gin.constant('StepLR',torch.optim.lr_scheduler.StepLR)  # noqa


@gin.configurable('train')
class TrainConfig(ConfigClass):

    def __init__(self,
                 epochs=gin.REQUIRED,
                 optimizer=torch.optim.Adam,
                 optimizer_args=gin.REQUIRED,
                 lr_schedule=torch.optim.lr_scheduler.StepLR,
                 lr_schedule_args=None,
                 batch_size=16,
                 gradient_acc_steps=1,
                 max_gradient_norm=1.0,
                 use_teacher_forcing=True):
        self.optimizer = optimizer
        self.optimizer_args = optimizer_args
        self.lr_schedule = lr_schedule
        self.lr_schedule_args = lr_schedule_args if lr_schedule_args else {'step_size': sys.maxsize}
        self.batch_size = batch_size
        self.gradient_acc_steps = gradient_acc_steps
        self.max_gradient_norm = max_gradient_norm
        self.epochs = epochs
        self.use_teacher_forcing = use_teacher_forcing

        self.test_optimizer_kwargs()
        self.test_lr_schedule_kwargs()

    def get_optimizer_kwargs(self,):
        # Determine the kwargs necessary in the optimizer init function
        args, varargs, varkw, *_ = inspect.getfullargspec(self.optimizer.__init__)
        return {k: v for k, v in self.optimizer_args.items() if k in args or k in varargs or k in varkw}

    def test_optimizer_kwargs(self,):
        signature = inspect.signature(self.optimizer.__init__)
        for elem in [k for k, v in signature.parameters.items() if v.default is inspect.Parameter.empty]:
            if elem not in self.optimizer_args and elem not in ('self', 'params'):
                raise ValueError('Key {} missing from required optimizer args'.format(elem))

    def get_lr_schedule_kwargs(self,):
        # Determine the kwargs necessary in the optimizer init function
        args, varargs, varkw, *_ = inspect.getfullargspec(self.lr_schedule.__init__)
        return {k: v for k, v in self.lr_schedule_args.items() if k in args or k in varargs or k in varkw}

    def test_lr_schedule_kwargs(self,):
        signature = inspect.signature(self.lr_schedule.__init__)
        for elem in [k for k, v in signature.parameters.items() if v.default is inspect.Parameter.empty]:
            if elem not in self.lr_schedule_args and elem not in ('self', 'optimizer'):
                raise ValueError('Key {} missing from required optimizer args'.format(elem))
