import inspect
import re

import ray
from ray import tune
from ray.tune import sample

_RAY_TUNE_SAMPLE_FNS = [s for s in inspect.getmembers(
    sample) if inspect.isfunction(s[1])]
RAY_TUNE_FUNCTION_REGEX_VALUES = [re.compile(
    r"(.*{}.*)\(.*\)".format(s[0])) for s in _RAY_TUNE_SAMPLE_FNS]


class Tunable():
    def __init__(self, value, path_string):
        self.value = value
        self.path_string = path_string


TUNABLES = {}


def _convert_tunables(input_object, regex_values, path_string=''):
    global TUNABLES
    if isinstance(input_object, list):
        output = list()
        for idx, elem in enumerate(input_object):
            output.append(_convert_tunables(elem, regex_values, path_string=path_string+'.'+f'item_{idx}'))
        return output
    if isinstance(input_object, dict):
        output = dict()
        for key, elem in input_object.items():
            output[key] = _convert_tunables(elem, regex_values, path_string=path_string+'.'+key)
        return output
    if isinstance(input_object, str):
        if any([r.match(input_object) for r in regex_values]):
            obj = Tunable(input_object, path_string)
            print(path_string)
            TUNABLES[path_string] = eval(input_object)
            return obj
    return input_object


def _replace_tunables(input_object, tunables):
    if isinstance(input_object, list):
        output = list()
        for elem in input_object:
            output.append(_replace_tunables(elem, tunables))
        return output
    if isinstance(input_object, dict):
        output = dict()
        for key, elem in input_object.items():
            output[key] = _replace_tunables(elem, tunables)
        return output
    if isinstance(input_object, Tunable):
        return tunables[input_object.path_string]
    return input_object


class ConfigClass():

    def get_config_dict(self,):
        return {k: str(v) for k, v in self.__dict__.items()}

    def process_tunables(self,map_path):
        global TUNABLES
        # Convert the tunables to the tunable class
        new_dict = {}
        for key, value in self.__dict__.items():
            new_dict[key] = _convert_tunables(
                value, RAY_TUNE_FUNCTION_REGEX_VALUES, path_string=map_path+'.'+key)
        self.__dict__ = new_dict

    def replace_tunables(self, tunables):
        new_dict = {}
        for key, value in self.__dict__.items():
            new_dict[key] = _replace_tunables(value, tunables)
        self.__dict__ = new_dict
