import torch

import sentencepiece as sp

from pele.nn.functional.util import padded_stack


class TokenizationPreprocessor(torch.nn.Module):

    def __init__(self,
                 vocab_file,
                 input_key,
                 output_key,
                 output_seqlen_key,
                 add_start_token=True,
                 add_stop_token=True,
                 start_token=1,
                 stop_token=2):
        super(TokenizationPreprocessor, self).__init__()
        self.input_key = input_key
        self.output_key = output_key
        self.output_seqlen_key = output_seqlen_key
        self.add_start_token = add_start_token
        self.add_stop_token = add_stop_token
        self.start_token = start_token
        self.stop_token = stop_token
        self.sp_model = sp.SentencePieceProcessor()
        self.sp_model.load(vocab_file)

    def add_start_stop(self, caption):
        if self.add_start_token:
            caption = [self.start_token] + caption
        if self.add_stop_token:
            caption = caption + [self.stop_token]
        return caption

    def forward(self, inputs):

        input_sentences = [x['caption'] for x in inputs[self.input_key]]
        tokenized_tensors = [torch.tensor(self.add_start_stop(self.sp_model.encode_as_ids(i))) for i in input_sentences]
        sentence_lengths = torch.tensor([len(s) for s in tokenized_tensors])

        # Stack and pad the captions
        tokenized = padded_stack(tokenized_tensors)

        # Check for CUDA
        if torch.cuda.is_available():
            tokenized = tokenized.cuda()
            sentence_lengths = sentence_lengths.cuda()

        return {
            self.output_key: tokenized,
            self.output_seqlen_key: sentence_lengths,
        }
