from typing import Any, Dict, List
import json
from pele.core.data.features import DataFeature


class JSONFeature(DataFeature):

    def __init__(self, json_path, output_key):
        self._json_path = json_path
        self._output_key = output_key

        # Load the JSON file
        with open(self._json_path, 'r') as json_file:
            self._data = json.load(json_file)

        # Save the feature metadata
        self._samples = self._data['samples']

    def keys(self,):
        return self._samples.keys()

    def shape(self,):
        return None

    def __getitem__(self, key):
        return {self._output_key: self._samples[key]}

    @classmethod
    def get_collate_fn(cls, output_key: str, **kwargs):  # pylint:disable=unused-argument

        def _collate(batch: List[Any]) -> Dict[str, Any]:
            return {output_key: batch}  # No direct tensor conversion for this dict

        return _collate
