from typing import Any, Dict, List

import yaml
try:
    from yaml import CLoader as Loader, CDumper as Dumper  # noqa
except ImportError:
    from yaml import Loader, Dumper  # noqa
    print('Falling back to Non-C-loader')

from pele.core.data.features import DataFeature


class YamlFeature(DataFeature):

    def __init__(self, yaml_path, output_key):
        self._yaml_path = yaml_path
        self._output_key = output_key

        # Load the YAML file
        with open(self._yaml_path, 'r') as yaml_file:
            self._data = yaml.load(yaml_file, Loader=Loader)

        # Save the feature metadata
        self._samples = self._data['samples']

    def keys(self,):
        return self._samples.keys()

    def shape(self,):
        return None

    def __getitem__(self, key):
        return {self._output_key: self._samples[key]}

    @classmethod
    def get_collate_fn(cls, output_key: str, **kwargs):  # pylint:disable=unused-argument

        def _collate(batch: List[Any]) -> Dict[str, Any]:
            return {output_key: batch}  # No direct tensor conversion for this dict

        return _collate
