from typing import Dict

import torch

from pele.config import LossConfig, TrainConfig, LogConfig
from pele.types import Input


class ExperimentLoss(torch.nn.Module):

    def __init__(self, loss_config: LossConfig, train_config: TrainConfig, logging_config: LogConfig) -> None:
        super(ExperimentLoss, self).__init__()
        self.losses = torch.nn.ModuleList(
            [l['type'](**{k: v for k, v in l.items() if k != 'type'}) for l in loss_config.losses])
        self.gradient_acc_steps = train_config.gradient_acc_steps
        self.max_gradient_norm = train_config.max_gradient_norm
        self.use_wandb = logging_config.use_wandb

    def forward(self, inputs: Input) -> Dict[str, torch.Tensor]:  # noqa
        return {'loss': torch.sum(torch.stack([l(inputs) for l in self.losses]))}
