import torch
import gin


class TestModel(torch.nn.Module):

    def __init__(self, context_dimension, internal_dimension, vocab_size, num_layers=2):
        super(TestModel, self).__init__()

        self.hidden_dimension = internal_dimension
        self.vocab_size = vocab_size
        self.num_layers = num_layers
        self.context_dimension = context_dimension

        # Model
        self.hidden_layer = torch.nn.Linear(2*self.hidden_dimension, self.hidden_dimension)

        # Projection Layer
        self.context_projection = torch.nn.Linear(self.context_dimension, self.hidden_dimension)
        self.projection_layer = torch.nn.Linear(self.hidden_dimension, self.vocab_size)

    def forward(self, inputs):

        context_projected = self.context_projection(inputs['context'])

        # Initial states
        input_tensors = torch.cat([
            context_projected.mean(dim=1).unsqueeze(1).expand(-1, inputs['caption_embedded'].shape[1], -1),
            inputs['caption_embedded'],
        ], dim=-1)

        # Output Decoder
        decoded = torch.nn.functional.relu(self.hidden_layer(input_tensors))
        text_logits = self.projection_layer(decoded)

        return {
            'predicted_caption_logits': text_logits,
        }
